@@ -479,8 +479,8 @@ int imap4Create( /*@notnull@ */ Pop3 pc, const char *const str)
 	/* special characters aren't allowed in hostnames, rfc 1034 */
 	const char *regexes[] = {
 		// type : username     :   password @ hostname (/ name)?(:port)?
-		".*imaps?:([^: ]{1,32}):([^@]{0,32})@([A-Za-z][-A-Za-z0-9_.]+)(/(\"[^\"]+\")|([^:@ ]+))?(:[0-9]+)?(  *([CcAaPp][-A-Za-z5 ]*))?$",
-		".*imaps?:([^: ]{1,32}) ([^ ]{1,32}) ([A-Za-z][-A-Za-z0-9_.]+)(/(\"[^\"]+\")|([^: ]+))?( [0-9]+)?(  *([CcAaPp][-A-Za-z5 ]*))?$",
+		".*imaps?:([^: ]{1,32}):([^@]{0,32})@([A-Za-z1-9][-A-Za-z0-9_.]+)(/(\"[^\"]+\")|([^:@ ]+))?(:[0-9]+)?(  *([CcAaPp][-A-Za-z5 ]*))?$",
+		".*imaps?:([^: ]{1,32}) ([^ ]{1,32}) ([A-Za-z1-9][-A-Za-z0-9_.]+)(/(\"[^\"]+\")|([^: ]+))?( [0-9]+)?(  *([CcAaPp][-A-Za-z5 ]*))?$",
 		NULL
 	};
 	char *unaliased_str;
@@ -571,7 +571,7 @@ int imap4Create( /*@notnull@ */ Pop3 pc, const char *const str)
 	IMAP_DM(pc, DEBUG_INFO, "authList= '%s'\n", PCU.authList);
 
 	if (strcmp(pc->action, "msglst") == 0 ||
-		strcmp(pc->notify, "msglst") == 0 ||
+		strcmp(pc->fetchcmd, "msglst") == 0 ||
 		strcmp(pc->button2, "msglst") == 0) {
 		PCU.wantCacheHeaders = 1;
 	} else {
@@ -642,7 +642,8 @@ authenticate_md5(Pop3 pc,
 	char buf[BUF_SIZE];
 	char buf2[BUF_SIZE];
 	unsigned char *md5;
-	GCRY_MD_HD gmh;
+	gcry_md_hd_t gmh;
+	gcry_error_t rc;
 
 	if (!strstr(capabilities, "AUTH=CRAM-MD5")) {
 		/* server doesn't support cram-md5. */
@@ -659,7 +660,11 @@ authenticate_md5(Pop3 pc,
 	strcpy(buf, PCU.userName);
 	strcat(buf, " ");
 	ask_user_for_password(pc, 0);
-	gmh = gcry_md_open(GCRY_MD_MD5, GCRY_MD_FLAG_HMAC);
+	rc = gcry_md_open(&gmh, GCRY_MD_MD5, GCRY_MD_FLAG_HMAC);
+	if (rc != 0) {
+		IMAP_DM(pc, DEBUG_INFO, "unable to initialize gcrypt md5\n");
+		return 0;
+	}
 	DEFROB(PCU.password);
 	gcry_md_setkey(gmh, PCU.password, strlen(PCU.password));
 	ENFROB(PCU.password);
