@@ -189,6 +189,8 @@ int shellCmdCheck(Pop3 pc)
 			pc->UnreadMsgs = 0;
 			pc->TotalMsgs = count_status;
 		}
+	} else if (strcasestr(commandOutput, "unable")) {
+		return -1;
 	} else if (sscanf(commandOutput, "%9s\n", pc->TextStatus) == 1) {
 		/* validate the string input */
 		int i;
@@ -243,8 +245,9 @@ struct msglst *shell_getHeaders( /*@notnull@ */ Pop3 pc)
 	}
 	return message_list;
 }
-void shell_releaseHeaders(Pop3 pc
-						  __attribute__ ((unused)), struct msglst *h)
+
+void
+shell_releaseHeaders(Pop3 pc __attribute__ ((unused)), struct msglst *h)
 {
 	for (; h != NULL;) {
 		struct msglst *n = h->next;
@@ -283,9 +286,19 @@ int shellCreate( /*@notnull@ */ Pop3 pc, const char *str)
 	}
 	commandline++;				/* shell:::> */
 
-	/* good thing strcpy handles overlapping regions */
+	/* strcpy is not specified to handle overlapping regions */
 	SH_DM(pc, DEBUG_INFO, "path= '%s'\n", commandline);
-	strcpy(pc->path, commandline);
+	{
+		char *tmp = strdup(commandline);
+		if (strlen(tmp) + 1 > BUF_BIG) {
+			SH_DM(pc, DEBUG_ERROR, "commandline '%s' is too long.\n",
+				  commandline);
+			memset(pc->path, 0, BUF_BIG);
+		} else {
+			strcpy(pc->path, tmp);
+		}
+		free(tmp);
+	}
 	return 0;
 }
 
