@@ -1,4 +1,4 @@
-/* $Id: wmbiff.c,v 1.67 2004/10/01 21:02:50 bluehal Exp $ */
+/* $Id: wmbiff.c,v 1.70 2005/10/07 03:07:58 bluehal Exp $ */
 
 // typedef int gcry_error_t;
 
@@ -23,6 +23,7 @@
 #include <X11/Xlib.h>
 #include <X11/xpm.h>
 #include <X11/cursorfont.h>
+#include <X11/keysym.h>
 
 #include <errno.h>
 #include <string.h>
@@ -38,6 +39,7 @@
 #include <dmalloc.h>
 #endif
 
+
 #include "wmbiff-master-led.xpm"
 static char wmbiff_mask_bits[64 * 64];
 static const int wmbiff_mask_width = 64;
@@ -210,6 +212,7 @@ struct path_demultiplexer {
 
 static struct path_demultiplexer paths[] = {
 	{"pop3:", pop3Create},
+	{"pop3s:", pop3Create},
 	{"shell:", shellCreate},
 	{"gicu:", gicuCreate},
 	{"licq:", licqCreate},
@@ -217,7 +220,6 @@ static struct path_demultiplexer paths[] = {
 	{"imap:", imap4Create},
 	{"imaps:", imap4Create},
 	{"sslimap:", imap4Create},
-	{"pop3:", pop3Create},
 	{"maildir:", maildirCreate},
 	{"mbox:", mboxCreate},
 	{NULL, NULL}
@@ -257,6 +259,8 @@ static int Read_Config_File(char *filename, int *loopinterval)
 		return 0;
 	}
 	while (!feof(fp)) {
+		/* skanky: -1 can represent an unparsed line 
+		   or an error */
 		if (ReadLine(fp, setting, value, &mbox_index) == -1)
 			continue;
 
@@ -611,6 +615,11 @@ static int wmbiffrc_permissions_check(const char *wmbiffrc_fname)
 								   problem: if you can't find it,
 								   neither can the bad guys.. */
 	}
+	if ((st.st_mode & S_IFDIR) != 0) {
+		DMA(DEBUG_ERROR, ".wmbiffrc '%s' is a directory!\n"
+			"exiting.  don't do that.", wmbiffrc_fname);
+		exit(EXIT_FAILURE);
+	}
 	if (st.st_uid != getuid()) {
 		char *user = getenv("USER");
 		DMA(DEBUG_ERROR,
@@ -1013,11 +1022,142 @@ static void restart_wmbiff(int sig
 	exit(EXIT_FAILURE);
 }
 
+extern int x_socket(void)
+{
+	return ConnectionNumber(display);
+}
+extern void ProcessPendingEvents(void)
+{
+	static int but_pressed_region = -1;	/* static so click can be determined */
+	int but_released_region = -1;
+	/* X Events */
+	while (XPending(display)) {
+		XEvent Event;
+		const char *press_action;
+
+		XNextEvent(display, &Event);
+
+		switch (Event.type) {
+		case Expose:
+			if (Event.xany.window != win && Event.xany.window != iconwin) {
+				msglst_redraw();
+			} else {
+				RedrawWindow();
+			}
+			break;
+		case DestroyNotify:
+			XCloseDisplay(display);
+			exit(EXIT_SUCCESS);
+			break;
+		case ButtonPress:
+			but_pressed_region =
+				CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
+			switch (Event.xbutton.button) {
+			case 1:
+				press_action = mbox[but_pressed_region].action;
+				break;
+			case 2:
+				press_action = mbox[but_pressed_region].button2;
+				break;
+			case 3:
+				press_action = mbox[but_pressed_region].fetchcmd;
+				break;
+			default:
+				press_action = NULL;
+				break;
+
+			}
+			if (press_action && strcmp(press_action, "msglst") == 0) {
+				msglst_show(&mbox[but_pressed_region],
+							Event.xbutton.x_root, Event.xbutton.y_root);
+			}
+			break;
+		case ButtonRelease:
+			but_released_region =
+				CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
+			if (but_released_region == but_pressed_region
+				&& but_released_region >= 0) {
+				const char *click_action, *extra_click_action = NULL;
+
+				switch (Event.xbutton.button) {
+				case 1:		/* Left mouse-click */
+					/* C-S-left will restart wmbiff. */
+					if ((Event.xbutton.state & ControlMask) &&
+						(Event.xbutton.state & ShiftMask)) {
+						restart_wmbiff(0);
+					}
+					/* do we need to run an extra action? */
+					if (mbox[but_released_region].UnreadMsgs == -1) {
+						extra_click_action =
+							mbox[but_released_region].actiondc;
+					} else if (mbox[but_released_region].UnreadMsgs > 0) {
+						extra_click_action =
+							mbox[but_released_region].actionnew;
+					} else {
+						extra_click_action =
+							mbox[but_released_region].actionnonew;
+					}
+					click_action = mbox[but_released_region].action;
+					break;
+				case 2:		/* Middle mouse-click */
+					click_action = mbox[but_released_region].button2;
+					break;
+				case 3:		/* Right mouse-click */
+					click_action = mbox[but_released_region].fetchcmd;
+					break;
+				default:
+					click_action = NULL;
+					break;
+				}
+				if (extra_click_action != NULL
+					&& extra_click_action[0] != 0
+					&& strcmp(extra_click_action, "msglst")) {
+					DM(&mbox[but_released_region], DEBUG_INFO,
+					   "runing: %s", extra_click_action);
+					(void) execCommand(extra_click_action);
+				}
+				if (click_action != NULL
+					&& click_action[0] != '\0'
+					&& strcmp(click_action, "msglst")) {
+					DM(&mbox[but_released_region], DEBUG_INFO,
+					   "running: %s", click_action);
+					(void) execCommand(click_action);
+				}
+			}
+
+			/* a button was released, hide the message list if open */
+			msglst_hide();
+
+			but_pressed_region = -1;
+			/* RedrawWindow(); */
+			break;
+		case MotionNotify:
+			break;
+		case KeyPress:{
+				XKeyPressedEvent *xkpe = (XKeyPressedEvent *) & Event;
+				KeySym ks = XKeycodeToKeysym(display, xkpe->keycode, 0);
+				if (ks > XK_0 && ks < XK_0 + min(9U, num_mailboxes)) {
+					const char *click_action = mbox[ks - XK_1].action;
+					if (click_action != NULL
+						&& click_action[0] != '\0'
+						&& strcmp(click_action, "msglst")) {
+						DM(&mbox[but_released_region], DEBUG_INFO,
+						   "running: %s", click_action);
+						(void) execCommand(click_action);
+					}
+				}
+
+			}
+			break;
+		default:
+			break;
+		}
+	}
+}
+
 static void do_biff(int argc, const char **argv)
 {
 	unsigned int i;
-	static int but_pressed_region = -1;	/* static so click can be determined */
-	int but_released_region = -1;
 	time_t curtime;
 	int Sleep_Interval;
 	const char **skin_xpm = NULL;
@@ -1081,112 +1221,8 @@ static void do_biff(int argc, const char **argv)
 
 		Sleep_Interval = periodic_mail_check();
 
-		/* X Events */
-		while (XPending(display)) {
-			XEvent Event;
-			const char *press_action;
+		ProcessPendingEvents();
 
-			XNextEvent(display, &Event);
-
-			switch (Event.type) {
-			case Expose:
-				if (Event.xany.window != win &&
-					Event.xany.window != iconwin) {
-					msglst_redraw();
-				} else {
-					RedrawWindow();
-				}
-				break;
-			case DestroyNotify:
-				XCloseDisplay(display);
-				exit(EXIT_SUCCESS);
-				break;
-			case ButtonPress:
-				but_pressed_region =
-					CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
-				switch (Event.xbutton.button) {
-				case 1:
-					press_action = mbox[but_pressed_region].action;
-					break;
-				case 2:
-					press_action = mbox[but_pressed_region].button2;
-					break;
-				case 3:
-					press_action = mbox[but_pressed_region].fetchcmd;
-					break;
-				default:
-					press_action = NULL;
-					break;
-
-				}
-				if (press_action && strcmp(press_action, "msglst") == 0) {
-					msglst_show(&mbox[but_pressed_region],
-								Event.xbutton.x_root,
-								Event.xbutton.y_root);
-				}
-				break;
-			case ButtonRelease:
-				but_released_region =
-					CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
-				if (but_released_region == but_pressed_region
-					&& but_released_region >= 0) {
-					const char *click_action, *extra_click_action = NULL;
-
-					switch (Event.xbutton.button) {
-					case 1:	/* Left mouse-click */
-						/* C-S-left will restart wmbiff. */
-						if ((Event.xbutton.state & ControlMask) &&
-							(Event.xbutton.state & ShiftMask)) {
-							restart_wmbiff(0);
-						}
-						/* do we need to run an extra action? */
-						if (mbox[but_released_region].UnreadMsgs == -1) {
-							extra_click_action =
-								mbox[but_released_region].actiondc;
-						} else if (mbox[but_released_region].
-								   UnreadMsgs > 0) {
-							extra_click_action =
-								mbox[but_released_region].actionnew;
-						} else {
-							extra_click_action =
-								mbox[but_released_region].actionnonew;
-						}
-						click_action = mbox[but_released_region].action;
-						break;
-					case 2:	/* Middle mouse-click */
-						click_action = mbox[but_released_region].button2;
-						break;
-					case 3:	/* Right mouse-click */
-						click_action = mbox[but_released_region].fetchcmd;
-						break;
-					default:
-						click_action = NULL;
-						break;
-					}
-					if (extra_click_action != NULL
-						&& extra_click_action[0] != 0
-						&& strcmp(extra_click_action, "msglst")) {
-						DM(&mbox[but_released_region], DEBUG_INFO,
-						   "runing: %s", extra_click_action);
-						(void) execCommand(extra_click_action);
-					}
-					if (click_action != NULL
-						&& click_action[0] != '\0'
-						&& strcmp(click_action, "msglst")) {
-						DM(&mbox[but_released_region], DEBUG_INFO,
-						   "running: %s", click_action);
-						(void) execCommand(click_action);
-					}
-				}
-
-				/* a button was released, hide the message list if open */
-				msglst_hide();
-
-				but_pressed_region = -1;
-				/* RedrawWindow(); */
-				break;
-			}
-		}
 		XSleep(Sleep_Interval);
 	}
 	while (forever);			/* forever is usually true,
